/* command line for TS program */

#include "ts_config.h"

#include <assert.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* default settings */
void config_reset (config *cfg)
{
	/* default config */
	cfg->fd_in = stdin;
	cfg->fd_out = stdout;
	
	cfg->verbose = 0;
	
	cfg->dump_pes = 0;
	cfg->dump_ts = 0;
	cfg->dump_es = 0;
	cfg->dump_info = 0;
	
	cfg->out_hex = 0;
	
	cfg->out = OUT_NONE;
	cfg->out_info_with_nit = 0;

	cfg->filter_audio = 0;
	cfg->filter_video = 0;
	cfg->filter_by_pid = 0;
	cfg->filtered_pid = 0;
}

/* parse config from command line */
int config_parse (config *cfg, int argc, char **argv)
{
	int i;
	memset(cfg->dev, 0x0, 64);
	assert (cfg);
	config_reset (cfg);
	if (1 == argc) {
	    dump_usage();
	    return -1;;
	}    

	for (i=1; i<argc; i++)
	{
		char *opt = argv[i];
		
		if (!strcmp (opt, "--verbose"))
		{
			(cfg->verbose)++;
		}
		else if (!strcmp (opt, "--dump-pes"))
		{
			cfg->dump_pes = 1;
		}
		else if (!strcmp (opt, "--dump-ts"))
		{
			cfg->dump_ts = 1;
		}
		else if (!strcmp (opt, "--dump-es"))
		{
			cfg->dump_es = 1;
		}
		else if (!strcmp (opt, "--dump-info"))
		{
			cfg->dump_info = 1;
		}
		else if (!strcmp (opt, "--dump"))
		{
			cfg->dump_ts = 1;
			cfg->dump_pes = 1;
			cfg->dump_es = 1;
			cfg->dump_info = 1;
		}
		else if (!strcmp (opt, "--hex"))
		{
			cfg->out_hex = 1;
		}
		else if (!strcmp (opt, "--es"))
		{
			cfg->out = OUT_ES;
		}
		else if (!strcmp (opt, "--pes"))
		{
			cfg->out = OUT_PES;
		}
		else if (!strcmp (opt, "--ts"))
		{
			cfg->out = OUT_TS;
		}
		else if (!strcmp (opt, "--info-xml"))
		{
			cfg->out = OUT_INFO_XML;
		}
		else if (!strcmp (opt, "--info-text"))
		{
			cfg->out = OUT_INFO_TEXT;
		}
		else if (!strcmp (opt, "--info-with-nit"))
		{
			cfg->out_info_with_nit = 1;
		}
		else if (!strcmp (opt, "--audio"))
		{
			cfg->filter_audio = 1;
		}
		else if (!strcmp (opt, "--video"))
		{
			cfg->filter_video = 1;
		}
		else if (!strcmp (opt, "--dev"))
		{
			memcpy(cfg->dev, argv[i+1], strlen(argv[i+1]));
			i++;
		}
		else if (isdigit((int)opt[0]))
		{ /* assume PID */
			cfg->filter_by_pid = 1;
			cfg->filtered_pid = atoi (opt); 
		}
		else
		{ /* usage */
		    dump_usage();			
			return -1;;
		}
	}
	
	if (0x0 == cfg->dev[0]) 
	{
	    dump_usage();
	    return -1;
	}    
	else 
	{    
	    return 0;
	}   
}

void dump_usage(void)
{
    fprintf (stderr, "./ts [--dev] [--audio] [--video] [--dump|--dump-pes|--dump-ts] [--es|--pes] [PID]\n");
#ifndef NDEBUG
	fprintf (stderr, "DEBUG build (assert enabled)\n");
#endif
    fprintf (stderr, "  --dev:           set the twinhan device node (/dev/twinhan0)\n");
	fprintf (stderr, "  --dump-ts:       output info on each TS packet on stderr\n");
	fprintf (stderr, "  --dump-pes:      output info on each PES packet on stderr\n");
	fprintf (stderr, "  --dump-es:       output info on each ES packet on stderr\n");
	fprintf (stderr, "  --dump-info:     output info on program information packets\n");
	fprintf (stderr, "  --dump:          output info on all packet types\n");
	fprintf (stderr, "  --info-xml:      dvbtune-like XML channel list\n");
	fprintf (stderr, "  --info-text:     short ascii channel list\n");
	fprintf (stderr, "  --info-with-nit: print info with multiplex (network) info (slow)\n");
	
	fprintf (stderr, "  --audio:         process only audio stream(s) (PES)\n");
	fprintf (stderr, "  --video:         process only video stream(s) (PES)\n");
	fprintf (stderr, "  --pes:           output raw PES packet on stdout\n");
	fprintf (stderr, "  --es:            output raw ES packet on stdout\n");
	fprintf (stderr, "  --ts:            output raw TS packet on stdout\n");
	fprintf (stderr, "  --hex:           output raw packets in hex\n");
}    
