#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <inttypes.h>
#include "read_demo.h"

#define DEV_NODE "/dev/twinhan0"
int main(int argc, char* argv[])
{    
    int nKSize = 5*1024;
    
    /****************** Init Device *********************/        
	PROGRAM_VALUE ProgramValue = 	    
	    //freq., BW, 0,vPid, aPid        (BW=6,7,8)
		{593000, 6, 0, 112, 114, 0,",,,",NULL,NULL};    //DTT -> CTS DTV1 
    
    CARD_VALUE cv;
    if (TH_ERROR == Getdevice_CardType(&cv))
	    return TH_ERROR;
	    
	if (TH_ERROR == Lock_Channel(&ProgramValue))
	    return TH_ERROR;
		
	SIGNAL_VALUE Signal_Val = {0, 0, 0};
	if (TH_ERROR == Getdevice_Signal_State(&Signal_Val))
	    return TH_ERROR;

	/************ Get TS stream => Demultiplexing => Save files ***********/		
	DEMULTIPLEXER_VALUE DemultiplexerValue;
	DemultiplexerValue.PID  = ProgramValue.vPID;  //for TH_PID_TS_PACKET, TH_PID_PES_PACKET, TH_ALL_PID_TS_PACKET
	DemultiplexerValue.vPID = ProgramValue.vPID;  //for TH_VA_TS_PACKET
	DemultiplexerValue.aPID = ProgramValue.aPID;  //for TH_VA_TS_PACKET
	DemultiplexerValue.StreamType = TH_PID_PES_PACKET;	
	
	
	if (TH_ERROR == Init_Demux(&DemultiplexerValue))
	    return TH_ERROR;	
    RET_MSG(">>>>>>>>>>>>> Begin to demux...Capture %d K bytes <<<<<<<<<<<<<<\n", nKSize);
    if (TH_ERROR == RecToFile("./ter.dat", nKSize))
        return TH_ERROR;	
    RET_MSG(">>>>>>>>>>>>>>>>>>>>>> Demux Completed <<<<<<<<<<<<<<<<<<<<<<<<<<\n");
    
	return  TH_NO_ERROR;
}

#define STREAM_BUFFER_SIZE 1024  //1K
int RecToFile(char *sFileName, int nKSize)
{    
    unsigned char pTemp[STREAM_BUFFER_SIZE];
        
    int nDevHandle = open(DEV_NODE, O_RDONLY);
    if (nDevHandle < 0) {
        ERR_MSG("No device finded!\n");
        return TH_ERROR;
    }  
    
    FILE *fd = fopen(sFileName, "wb");
    if (fd == NULL) {
        ERR_MSG("open %s err!! \n", sFileName);
        return TH_ERROR;
    }
    
    int i;    
    for (i=0; i<nKSize; i++) {
        //memset((void*)pTemp, 0xF0, STREAM_BUFFER_SIZE);
        if (read(nDevHandle, pTemp, STREAM_BUFFER_SIZE) <= 0) {
          ERR_MSG("read() err!! \n");          
          fclose(fd);
          close(STREAM_BUFFER_SIZE);
          return TH_ERROR;
        }
        fwrite(pTemp, STREAM_BUFFER_SIZE, 1, fd);
    }    
    
    fclose(fd);
    close(nDevHandle);  
    return TH_NO_ERROR;
}

int Init_Demux(P_DEMULTIPLEXER_VALUE pDMV)
{
    int nDevHandle = open(DEV_NODE, O_RDONLY);
    if (nDevHandle < 0) {
        ERR_MSG("No device finded!\n");
        return TH_ERROR;
    }   
    
    if (ioctl(nDevHandle, INIT_DEMUX, pDMV)) {
        ERR_MSG("ioctl READ_ENABLE err !!\n");
        return TH_ERROR;
    }
    
    close(nDevHandle);     
    DBG_MSG("Read_Enable OK !\n");
    return  TH_NO_ERROR;
}
    
int Getdevice_Signal_State(P_SIGNAL_VALUE pSignal_Val)
{
    int nDevHandle = open(DEV_NODE, O_RDONLY);
    if (nDevHandle < 0) {
        ERR_MSG("No device finded!\n");
        return TH_ERROR;
    }   
    
    if (ioctl(nDevHandle, DST_GET_SIGNAL, pSignal_Val)) {
        ERR_MSG("ioctl DST_GET_SIGNAL failed !\n");
        return TH_ERROR;
    }    
	
    DBG_MSG("Device signal: LockFlag=%d,  strength=%d,  quality=%d\n", 
             pSignal_Val->nLockFlag, pSignal_Val->nDevSigStrength, pSignal_Val->nDevSigQuality);
    close(nDevHandle);  
    return TH_NO_ERROR;
}

int Getdevice_CardType(P_CARD_VALUE pCV)
{
    int nDevHandle = open(DEV_NODE, O_RDONLY);
    if (nDevHandle < 0) {
        ERR_MSG("No device finded!\n");
        return TH_ERROR;
    }
    
    if (ioctl(nDevHandle, DST_GET_CARD_TYPE, pCV )) {
        ERR_MSG("ioctl DST_GET_CARD_TYPE failed !\n");
        return TH_ERROR;
    }
    
    close(nDevHandle);     
    DBG_MSG("Card Name=%s,  TS_Size=%d !\n", pCV->sCard_Name, pCV->nTS_SIZE);
    return  TH_NO_ERROR;    
}    

int Lock_Channel(P_PROGRAM_VALUE  pProgramValue)
{
    int nDevHandle = open(DEV_NODE, O_RDONLY);
    if (nDevHandle < 0) {
		ERR_MSG("No device finded!\n");
		return TH_ERROR;
	}
	
    if(ioctl(nDevHandle, TUNER_SETTUNER, pProgramValue ) ) {
        ERR_MSG("ioctl Set tuner failed !\n");
        return TH_ERROR;
    }
    if (ioctl(nDevHandle, DST_START_CAPTURE, 0 ) ) {
        ERR_MSG("ioctl start capture failed!\n");
        return TH_ERROR;
    }
    
    close(nDevHandle);        
    DBG_MSG("Lock channel OK!\n");
    return TH_NO_ERROR;
}

/*******************************************************************************/
